#ifndef _HitROOT_h_
#define _HitROOT_h_

#include <TObject.h>
#include <TRef.h>
#include <TVector3.h>
#include <RConfigure.h>
#include <iostream>

class Hit;

class HitROOT : public TObject {

public:

  friend class Hit;
  friend class Write2ROOT;
  friend class StationROOT;

  friend std::ostream &operator<<(std::ostream&, const HitROOT*);

public:
  HitROOT();

private:
  HitROOT(const HitROOT &);
  ~HitROOT();

  void Clear(Option_t* /*option*/ = "");

public:  
  const HitROOT& operator=(const HitROOT&);
  int operator==(const HitROOT&) const;
  
  void Print(Option_t *option="") const;
  void Dump() const;

  inline Double_t GetEnergy() const {return fEnergy;}
  inline Double_t GetTime() const {return fTime;}
  inline Double_t GetCharge() const {return fCharge;}
  inline Double_t GetWeight() const {return fWeight;}

  inline TVector3 GetPosition() const {return fPosition;}
  inline TVector3 GetDirection() const {return fDirection;} 
  inline TVector3 GetLocalPosition() const {return fLocalPosition;}
  
  inline Int_t GetPDG() const {return fPDG;}     
  inline Int_t GetDetectorID() const {return fDetectorID;}   

private:

  Double_t fEnergy;
  Double_t fTime;
  Double_t fCharge;
  Double_t fWeight;

  TVector3 fPosition; 
  TVector3 fDirection;
  TVector3 fLocalPosition;

  Int_t fPDG;
  Int_t fDetectorID;

public:
    ClassDef(HitROOT,2)
};

#endif
