#include "Hit.hh"

#include "Write2ROOT.hh"
#include "PrimaryGeneratorAction.hh"
#include "HitROOT.hh"

#include <TMap.h>
#include <TFile.h>
#include <TTree.h>
#include <TDirectory.h>
#include <TMath.h>
#include <iostream>
#include <string>

#include "G4RunManager.hh"
#include "G4SystemOfUnits.hh"
#include "G4GeneralParticleSource.hh"
#include "G4PrimaryParticle.hh"
#include "G4PrimaryVertex.hh"
#include "G4ParticleDefinition.hh"

using namespace std;

Write2ROOT::Write2ROOT() : 
  fRootFile(NULL),
  fTree(NULL),
  fRootFilename("")
{
}


Write2ROOT::Write2ROOT(const string& name) :
  fRootFile(0),
  fTree(NULL),
  fRootFilename(name)
{
  Open(fRootFilename);
}


Write2ROOT::~Write2ROOT()
{
  SaveAndClose();
}


void Write2ROOT::Open(const string& filename)
{
  TDirectory *tmpSaveDir = gDirectory;
  fRootFile = new TFile(filename.c_str(), "RECREATE");
  fSaveDir=gDirectory;
  tmpSaveDir->cd();
  CreateTree();
}


void Write2ROOT::SaveAndClose()
{
  if (fRootFile) {
    if (fRootFile->IsOpen()) {
      TDirectory *tmpSaveDir = gDirectory;
      fSaveDir->cd();
      fTree->Write("",TObject::kOverwrite);
      fRootFile->Close();
      tmpSaveDir->cd(); 
      fTree  = NULL;
    }
  }

  if (fRootFile) {
    delete fRootFile; 
    fRootFile = NULL;
  }
}


void Write2ROOT::CreateTree()
{
  TDirectory *tmpSaveDir = gDirectory;
  fSaveDir->cd();
 
  fEventROOT = new EventROOT(); 
  fTree = new TTree("Tree","Tree of Hits");
  fTree->Branch("EventROOT","EventROOT",&fEventROOT);
  gDirectory->ls();
  tmpSaveDir->cd();
  gDirectory->ls();
}


void 
Write2ROOT::AddHit(const Hit & hit)
{
  HitROOT hitROOT;
  hit.Fill(hitROOT);
  fEventROOT->AddHit(hitROOT);
}


void 
Write2ROOT::SetEventNumber(const int & i)
{
  fEventROOT->fEventNumber = i;
}


void Write2ROOT::FillEventSummary()
{
  //static G4RunManager * runMan = G4RunManager::GetRunManager();

  // const PrimaryGeneratorAction * primGen = 
  // dynamic_cast<const PrimaryGeneratorAction*>(runMan->GetUserPrimaryGeneratorAction());
  // const DetectorConstruction * detector = 
  // dynamic_cast<const DetectorConstruction*>(runMan->GetUserDetectorConstruction());

}


void 
Write2ROOT::SetPrimaryInfo(const PrimaryGeneratorAction * pGen)
{
  fEventROOT->fPrimaryType = pGen->GetParticleDefinition()->GetPDGEncoding();
  fEventROOT->fPrimaryEnergy = pGen->GetParticleEnergy()/CLHEP::GeV;
  G4ThreeVector p = pGen->GetParticlePosition();
  G4ThreeVector dir = pGen->GetParticleMomentumDirection();

  fEventROOT->fPrimaryPosition = TVector3(p.x()/CLHEP::m,p.y()/CLHEP::m,p.z()/CLHEP::m);

  fEventROOT->fPrimaryDirection = TVector3(dir.x(),dir.y(),dir.z());
}


void 
Write2ROOT::FillTree()
{
  WriteIt();
}


void 
Write2ROOT::WriteIt()
{
  fTree->Fill();
}


void 
Write2ROOT::ClearEvent()
{
  if (fEventROOT)
    fEventROOT->Reset();
}


void 
Write2ROOT::PrintSummary()
const
{
  fEventROOT->Print();
}

