//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
/// \file persistency/gdml/G02/include/G02DetectorConstruction.hh
/// \brief Definition of the G02DetectorConstruction class
//
//
// $Id: G02DetectorConstruction.hh 68025 2013-03-13 13:43:46Z gcosmo $
//
// Class G02DetectorConstruction
//
// A detector construction class loading the geometry from GDML files.
//
// ----------------------------------------------------------------------------

#ifndef DetectorConstruction_H
#define DetectorConstruction_H 1


#include "G4VUserDetectorConstruction.hh"
#include "G4PhysicalVolumeStore.hh"
#include "G4GDMLParser.hh"

#include "G4Material.hh"

#include "globals.hh"

class DetectorMessenger;

// ----------------------------------------------------------------------------

/// Detector construction used in GDML read/write example

class DetectorConstruction : public G4VUserDetectorConstruction
{
  public:

    // Constructor and destructor
    //
    DetectorConstruction();
   ~DetectorConstruction();

    // Construction of SubDetectors
    //
    virtual G4VPhysicalVolume* Construct();
    virtual void ConstructSDandField();


    // Make List of materials
    //
    void ListOfMaterials();

    // Writing and Reading GDML
    //
    void SetReadFile( const G4String& File );
    void SetWriteFile( const G4String& File );

    // Reading STEP File
    //
    void SetStepFile( const G4String& File );

    // G4ThreeVector GetPosition(const G4String& name) const {
    //     G4PhysicalVolumeStore * store = G4PhysicalVolumeStore::GetInstance();
    //     G4VPhysicalVolume * physVol = store->GetVolume(name);
    //     G4ThreeVector pos;
    //     while (physVol->GetName() != fWorldPhysVol->GetName())
    //     {
    //         pos += physVol->GetObjectTranslation();
    //         physVol = store->GetVolume(physVol->GetMotherLogical()->GetName());
    //     }
    //     return pos;
    // }
    
  private:

    G4VPhysicalVolume* fWorldPhysVol;
    G4Material* fAir ;
    G4Material* fAluminum ;
    G4Material* fPb;
    G4Material* fXenon;

    // GDMLparser
    //
    G4GDMLParser fParser;
        
    // Reading and Writing Settings
    //
    G4String fReadFile;
    G4String fWriteFile;
    G4String fStepFile;
    G4int fWritingChoice;

    // Detector Messenger
    //
    DetectorMessenger* fDetectorMessenger;
      
    // World Dimentions
    //
    G4double fExpHall_x; 
};

// ----------------------------------------------------------------------------

#endif
