#ifndef _Hit_h_
#define _Hit_h_ 1

#include "G4VHit.hh"
#include "G4THitsCollection.hh"
#include "G4Allocator.hh"
#include "G4ThreeVector.hh"

class HitROOT;

class Hit : public G4VHit {

  public:

    friend class SensitiveDetector;

    Hit();
    Hit(const Hit&);
    ~Hit();

    const Hit& operator=(const Hit&);
    int operator==(const Hit&) const;
  
    inline void* operator new(size_t);
    inline void  operator delete(void*);

    void Draw();
    void Print();

  private:
    G4double fEnergy;            
    G4double fTime;           
    G4double fCharge;
    G4double fWeight;
    G4ThreeVector fWorldPosition;  // Position of the hit in world coordinates
    G4ThreeVector fPosition;       // Position of the hit in local coordinates
    G4ThreeVector fDirection;       // Direction of the hit
    G4int fPDG;                 // PDG code identifying particle that produced the hit; 
    G4int fDetectorID;    // Code identifying the hit detector

public:
    inline G4double GetEnergy() const {return fEnergy;}
    inline G4double GetTime() const {return fTime;}
    inline G4double GetCharge() const {return fCharge;} 
    inline G4double GetWeight() const {return fWeight;} 
    inline G4ThreeVector GetPosition() const {return fPosition;} 
    inline G4ThreeVector GetWorldPosition() const {return fWorldPosition;}
    inline G4ThreeVector GetDirection() const {return fDirection;} 
    inline G4int GetPDG() const {return fPDG;} 
    inline G4int GetDetectorID() const {return fDetectorID;}

    void Fill(HitROOT &) const;

private:
    inline void SetEnergy(const G4double & En) {fEnergy = En;}
    inline void SetTime(const G4double & Ti) {fTime = Ti;}
    inline void SetCharge(const G4double & ch) {fCharge = ch;}
    inline void SetWeight(const G4double & w) {fWeight = w;}
    inline void SetPosition(const G4ThreeVector & xyz) {fPosition = xyz;}
    inline void SetWorldPosition(const G4ThreeVector & xyz) {fWorldPosition = xyz;}
    inline void SetDirection(const G4ThreeVector & xyz) {fDirection = xyz;}
    inline void SetPDG(const G4int & pdg) {fPDG = pdg;}    
    inline void SetDetectorID(const G4int & id) {fDetectorID = id;} 
  };

  typedef G4THitsCollection<Hit> HitsCollection;

extern G4Allocator<Hit> HitAllocator;


  inline void* Hit::operator new(size_t)
  {
    void* aHit;
    aHit = (void*) HitAllocator.MallocSingle();
    return aHit;
  }

  inline void Hit::operator delete(void* aHit)
  {
    HitAllocator.FreeSingle((Hit*) aHit);
  }
#endif
