//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
/// \file persistency/gdml/G02/include/RunAction.hh
/// \brief Definition of the RunAction class
//
//
// $Id: RunAction.hh 68025 2013-03-13 13:43:46Z gcosmo $
//
// Class RunAction
//
// Simple run action class.
//
// ----------------------------------------------------------------------------

#ifndef RunAction_h
#define RunAction_h 1

#include <iostream>
#include <fstream>

#include "globals.hh"
#include "G4UserRunAction.hh"

class Write2ROOT;
class G4Run;

// ----------------------------------------------------------------------------

/// Run action used in GDML read/write example

class RunAction : public G4UserRunAction
{
	friend class EventAction;

  public:

    RunAction();
    RunAction(Write2ROOT *);   
   ~RunAction();

    virtual void BeginOfRunAction(const G4Run*);
    virtual void EndOfRunAction(const G4Run*);

    const std::fstream & GetOutputFile(void) const {return fOutFile;}
    std::fstream & GetOutputFile(void) {return fOutFile;}

    private:
    	Write2ROOT *fWrite2ROOT;
    	std::fstream fOutFile;       
};

// ----------------------------------------------------------------------------

#endif
