//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
#include "RunAction.hh"
#include "EventAction.hh"
#include "PrimaryGeneratorAction.hh"
#include "Hit.hh"
#include "Write2ROOT.hh"

#include "G4RunManager.hh"
#include "G4SDManager.hh"
#include "G4UImanager.hh"
#include "G4StateManager.hh"
#include "G4ios.hh"

using namespace std;

EventAction::EventAction():
  G4UserEventAction(),
  fWrite2ROOT(0),
  fNEvents(0)
{}


EventAction::~EventAction()
{}


void 
EventAction::BeginOfEventAction(const G4Event* /*event*/)
{
  static G4RunManager * runMan = G4RunManager::GetRunManager();
  static const RunAction * runAction = dynamic_cast<const RunAction*>(runMan->GetUserRunAction());
  
  fNEvents++;

  fWrite2ROOT = runAction->fWrite2ROOT;
}


void EventAction::EndOfEventAction(const G4Event* event)
{
  if (!fWrite2ROOT) 
    return;

  static G4RunManager * runMan = G4RunManager::GetRunManager();
  
  fWrite2ROOT->ClearEvent();
  fWrite2ROOT->SetEventNumber(event->GetEventID()); 

  const PrimaryGeneratorAction * primGen =
  dynamic_cast <const PrimaryGeneratorAction *>(runMan->GetUserPrimaryGeneratorAction());
  fWrite2ROOT->SetPrimaryInfo(primGen);

  // Copy hits to ROOT tree...

  G4HCofThisEvent* HCE = event->GetHCofThisEvent();

  HitsCollection * Hits = 0;
  Hits = GetHitsCollection(HCE,"Hits"); // Particles detected at detector
  if (Hits) {
    const G4int nHits = Hits->entries(); 
    for (G4int iHit = 0; iHit<nHits; ++iHit) 
      fWrite2ROOT->AddHit(*((*Hits)[iHit]));
  } 
  else 
    G4Exception("EndOfEventAction","",JustWarning,"Hits collection Hits not found."); 

  fWrite2ROOT->FillTree();
  
  RunAction * runAction = (RunAction*)(runMan->GetUserRunAction());
  std::fstream & OutFile = runAction->GetOutputFile();

  OutFile << fWrite2ROOT->GetEvent();
}


HitsCollection * EventAction::GetHitsCollection(G4HCofThisEvent* HCE, const G4String & name)
{
  static G4SDManager * SDman = G4SDManager::GetSDMpointer(); 
  // SDman->ListTree();
  // G4HCtable* hct = SDman->GetHCtable();
  // for (G4int i = 0; i<hct->entries(); ++i)
  //   G4cout << i << "  " << hct->GetSDname(i) << "  " << hct->GetHCname(i) << G4endl;

  const G4int HitsCollID  = SDman->GetCollectionID(name); 
  if (HCE && HitsCollID > -1)   
    return (HitsCollection*)(HCE->GetHC(HitsCollID)); 
  else
    return 0;
}


