#include "Hit.hh"
#include "HitROOT.hh"
#include "G4VVisManager.hh"
#include "G4Colour.hh"
#include "G4Circle.hh"
#include "G4VisAttributes.hh"
#include "G4UnitsTable.hh"
#include "G4SystemOfUnits.hh"

G4Allocator<Hit> HitAllocator;

Hit::Hit() : 
  fEnergy(0.0),
  fTime(0.0),
  fCharge(0.0),
  fWeight(0.0),
  fWorldPosition(0.0),
  fPosition(0.0),
  fDirection(0.0),
  fPDG(0),
  fDetectorID(0)
{
}


Hit::~Hit()
{
}


Hit::Hit(const Hit& right) : G4VHit()
{
  fEnergy = right.fEnergy;
  fTime = right.fTime;
  fCharge = right.fCharge;
  fWeight = right.fWeight;
  fWorldPosition = right.fWorldPosition;
  fPosition = right.fPosition;
  fDirection = right.fDirection;
  fPDG = right.fPDG;
  fDetectorID = right.fDetectorID;
}


const Hit& Hit::operator=(const Hit& right)
{
  fEnergy = right.fEnergy;
  fTime = right.fTime;
  fCharge = right.fCharge;
  fWeight = right.fWeight;
  fWorldPosition = right.fWorldPosition;
  fPosition = right.fPosition;
  fDirection = right.fDirection;
  fPDG = right.fPDG;
  fDetectorID = right.fDetectorID;
 
  return *this;
}


int Hit::operator==(const Hit& right) const
{
  return (this==&right) ? 1 : 0;
}


void 
Hit::Fill(HitROOT & hit) 
const
{
  hit.fEnergy = fEnergy/GeV;
  hit.fTime = fTime/ns;
  hit.fCharge = fCharge;
  hit.fWeight = fWeight;

  G4ThreeVector p;

  p = fWorldPosition;
  hit.fPosition.SetXYZ(p.x()/m,p.y()/m,p.z()/m);

  p = fPosition;
  hit.fLocalPosition.SetXYZ(p.x()/m,p.y()/m,p.z()/m);

  p = fDirection;
  hit.fDirection.SetXYZ(p.x(),p.y(),p.z());

  hit.fPDG = fPDG;
  hit.fDetectorID = fDetectorID;  
}

void Hit::Draw()
{
  G4VVisManager* pVVisManager = G4VVisManager::GetConcreteInstance(); 
  if(pVVisManager) 
    { 
      // define a circle in a 3D space 
      G4double tMin = 0*ns;
      G4double tMax = 20000*ns;
      G4double iRed   = 0; 
      G4double iBlue  = ((tMax-fTime)/(tMax-tMin)); 
      G4double iGreen = 0; 

      G4Circle circle(fWorldPosition); 
      //  circle.SetScreenSize(1.0); 

      circle.SetWorldSize(10*mm); 
      // circle.SetScreenDiameter(2.0); 
      circle.SetFillStyle(G4Circle::filled); 
      // make the circle red 
      G4Colour colour(iRed,iGreen,iBlue); 
      // G4Colour colour(0,200,0); 
      // G4VisAttributes attribs(FDsimG4Colours::yellow); 

      G4VisAttributes attribs(colour); 
      circle.SetVisAttributes(attribs); 
      pVVisManager->Draw(circle);
    }
}


void Hit::Print()
{
  G4cout << "*****************************************" << G4endl;
  G4cout << "                 HIT                      " << G4endl;
  G4cout << "  Energy (GeV)         :       " << fEnergy/GeV   << G4endl;
  G4cout << "  Local Position  :       " << G4BestUnit(fPosition,"Length")   << G4endl;
  G4cout << "  World Position  :       " << G4BestUnit(fWorldPosition,"Length")   << G4endl;
  G4cout << "  Time (ns)           :       " << fTime/ns  << G4endl;
  G4cout << "  PDG code              :       " << fPDG      << G4endl;
  G4cout << "  Weight              :       " << fWeight       << G4endl;
  G4cout << "*****************************************" << G4endl;
}
