//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
/// \file persistency/gdml/G02/src/G02PrimaryGeneratorAction.cc
/// \brief Implementation of the G02PrimaryGeneratorAction class
//
//
// $Id: G02PrimaryGeneratorAction.cc 68025 2013-03-13 13:43:46Z gcosmo $
//
// Class G02PrimaryGeneratorAction implementation
//
// ----------------------------------------------------------------------------

#include "TH1D.h"
#include "TFile.h"

#include "PrimaryGeneratorAction.hh"
#include "DetectorConstruction.hh"

#include "globals.hh"
#include "G4RunManager.hh"
#include "G4VUserDetectorConstruction.hh"
#include "G4ParticleDefinition.hh"
#include "G4SystemOfUnits.hh"

using namespace std;


PrimaryGeneratorAction::PrimaryGeneratorAction() : G4VUserPrimaryGeneratorAction(),
  fParticleTable(0),
  fParticleGun(0),
  fUseGPS(kTRUE)  
{
  // Define the particle gun and fetch the pointer to the particle table 
  //
  fParticleGun = new G4GeneralParticleSource();
  fParticleTable = G4ParticleTable::GetParticleTable();

// Set default particle
  fParticleGun->SetParticleDefinition(fParticleTable->FindParticle("mu-"));

// *** Use the function below to set the muon energy spectrum for a given cos(theta) - change Eth in log10(Eth) ->now: log10(8.0(GeV))
  SetEnergyDistribution(cos(30.0*deg),log10(8.0),3.0,30);  //cos(30) is the angle to calculate the energy distribution (different from the muons entry angle in the simulation)
}


PrimaryGeneratorAction::~PrimaryGeneratorAction()
{
  delete fParticleGun;
}


void PrimaryGeneratorAction::GeneratePrimaries(G4Event* event)
{
 // if (fUseGPS)
    GeneratePrimariesFromGPS(event);
  // else
  //   GeneratePrimariesFromUser(event);
}


void PrimaryGeneratorAction::GeneratePrimariesFromGPS(G4Event* event)
{
  fParticleGun->GeneratePrimaryVertex(event);
  fVertex = event->GetPrimaryVertex();
}

// In principle not to be used anymore; now handled in TrackingAction.
// void PrimaryGeneratorAction::GeneratePrimariesFromUser(G4Event* event)
// {
//   G4PhysicalVolumeStore * store = G4PhysicalVolumeStore::GetInstance();
//   G4VPhysicalVolume * physDet = store->GetVolume("Detector");
//   G4VPhysicalVolume * physEarth = store->GetVolume("Earth");

//   G4Box * box  = dynamic_cast<G4Box*>(physDet->GetLogicalVolume()->GetSolid());

//   G4double x = box->GetXHalfLength();
//   G4double y = box->GetYHalfLength();
//   G4double detRadius = sqrt(x*x + y*y);


//   G4ThreeVector detPosition = physDet->GetObjectTranslation() + physEarth->GetObjectTranslation(); 

//   G4SPSEneDistribution *eneDist  = fParticleGun->GetCurrentSource()->GetEneDist();
//   G4SPSAngDistribution *angDist  = fParticleGun->GetCurrentSource()->GetAngDist();
//   G4SPSPosDistribution *posDist  = fParticleGun->GetCurrentSource()->GetPosDist();

//   G4double kineticE = eneDist->GenerateOne(fParticleGun->GetCurrentSource()->GetParticleDefinition());

//   G4ParticleDefinition * partDef = fParticleGun->GetCurrentSource()->GetParticleDefinition();

//   // Compute the multiplescatttering rms (from PDG)
//   G4Material * earth = physEarth->GetLogicalVolume()->GetMaterial();
//   const G4double X0 = earth->GetRadlen();
//   const G4double depth = abs(detPosition.z());
//   const G4double mass = partDef->GetPDGMass();
//   const G4double charge = partDef->GetPDGCharge();
//   const G4double charge2 = charge*charge;

//   // Kinematical variables
//   const G4double energy = kineticE + mass;
//   const G4double energy2 = energy*energy;
//   const G4double momentum2 = energy2 - mass*mass;
//   const G4double betapc = momentum2/energy;
//   const G4double beta2 = momentum2/energy2;

//   G4ThreeVector pos, newpos, dir;
//   G4double lambda;

// // Find a generated position + direction that hits the detector, accounting for the multiple scattering
//   while (true) {
//     pos = posDist->GenerateOne();
//     dir = angDist->GenerateOne();

//   // Compute track position at detector

//     lambda = (detPosition.z()-pos.z())/dir.z();
//     newpos = pos + lambda * dir;

//     // Compute multiple scattering 
//     const G4double slantDepth = abs(depth/cos(dir.theta()));

//     G4double theta0;
//     if (charge  == 0.0) {
//       theta0 = 0.0;
//     }
//     else {
//       theta0 = 13.6*MeV/betapc * abs(charge) * sqrt(slantDepth/X0) * (1.0 + 0.038*log(slantDepth/X0*charge2/beta2)); 
//     }

//   // Deviation due to multiple scattering
//     const G4double R0 = slantDepth * tan(theta0);

//   //G4cout << X0/cm << "   " << slantDepth/cm << "  " << mass/MeV << "   " << beta2 << "   " << theta0*180.0/CLHEP::pi << "   " << R0/m << endl;

//     if ( (newpos-detPosition).perp() < detRadius + 2.0*R0) 
//       break;
//   }

// // Let's create the vertex
// //  G4cout << detRadius/m << "   " <<  detPosition/m << "    " <<  dir << "    " <<  pos/m << "   " <<  newpos/m << G4endl;

//     fVertex = new G4PrimaryVertex(pos,0.0*ns);
//     G4PrimaryParticle* particle = new G4PrimaryParticle(fParticleGun->GetCurrentSource()->GetParticleDefinition());
//     particle->SetKineticEnergy(kineticE);
//     particle->SetMomentumDirection(dir);
//     fVertex->SetPrimary(particle);
//     event->AddPrimaryVertex(fVertex);
// }


void PrimaryGeneratorAction::SetEnergyDistribution(G4double Costh,G4double logEmin,G4double logEmax,G4int Nbins)  //define energy distribution, parameters are indicated above in ***
{
  G4SPSEneDistribution *eneDist = fParticleGun->GetCurrentSource()->GetEneDist();
  eneDist->SetEnergyDisType("Arb");


  G4ThreeVector EnergyHisto;
  G4double energy, prob;
  
  G4cout << "Generating spectrum for costh = " <<  Costh << G4endl;
  
  G4double logE;
  G4double delta = (logEmax-logEmin)/float(Nbins);

  G4double norm = dNdEdOmegadtdA(pow(10.0,logEmin)*GeV,Costh);

  G4cout << norm << G4endl;

  for (int ibin=0;ibin<=Nbins;++ibin) {      // bin size

    logE = logEmin + ibin*delta;
    energy = pow(10.0,logE)*GeV;
    prob = dNdEdOmegadtdA(energy,Costh)/norm;

    if (prob > 0.0) {                                //energy probability
      EnergyHisto = G4ThreeVector(energy,prob,0.0);
      eneDist->ArbEnergyHisto(EnergyHisto);
      G4cout <<  EnergyHisto.x()/GeV << " (GeV)     " << EnergyHisto.y() <<  G4endl;
    }
  }

  eneDist->ArbInterpolate("Exp");
}


// Muon Flux Distribution Equation (at sea level)

G4double PrimaryGeneratorAction::dNdEdOmegadtdA(G4double energy, G4double costh)
{
 //units of a*b*c : cm-2 s-1 sr-1
  //double costh=cos(TMath::DegToRad()*theta);

  const G4double E = energy/GeV;
  
  double a=18/(E*costh+145);
  double b=pow(E+2.7/costh,-2.7);
  double c=(E+5)/(E+5/costh);

  //  double d=pow(p,2.7);
  //conversion from cm^-2 to  m^-2    *1E4
  //conversion from sr^-1   to  deg^-2    *3E-4
  //conversion from sr^-1   to  (0.001*0.001) sr^-1    *1E-6
  //conversion from s^-1    to day^-1 *86400

  return a*b*c*1E4;//*3E-4;  // return units  m-2 s-1 sr-1
}
