//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
/// \file persistency/gdml/G04/src/SensitiveDetector.cc
/// \brief Implementation of the SensitiveDetector class
//
//
// $Id: SensitiveDetector.cc 68025 2013-03-13 13:43:46Z gcosmo $
//

#include "SensitiveDetector.hh"
#include "Hit.hh"
#include "G4RunManager.hh"
#include "G4HCofThisEvent.hh"
#include "G4Step.hh"
#include "G4ThreeVector.hh"
#include "G4SDManager.hh"
#include "G4ios.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

SensitiveDetector::SensitiveDetector(const G4String& name)
  : G4VSensitiveDetector(name)
{
	collectionName.insert("Hits");
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

SensitiveDetector::~SensitiveDetector()
{
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void SensitiveDetector::Initialize(G4HCofThisEvent* HCoE)
{
	static int HCID0 = -1;
	fHitsCollection = new HitsCollection(SensitiveDetectorName,collectionName[0]);  

	if (HCID0 < 0) 
		HCID0 = GetCollectionID(0);


	HCoE->AddHitsCollection(HCID0,fHitsCollection);
}


//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
// G4bool SensitiveDetector::ProcessHits(G4Step*, G4TouchableHistory*)
// {
//   G4cout << "Hit." << G4endl;
//   return true;
// }


G4bool SensitiveDetector::ProcessHits(G4Step* aStep, G4TouchableHistory*)
{

	G4StepPoint* preStepPoint = aStep->GetPreStepPoint();
  const G4Track* track = aStep->GetTrack();

//  	G4VPhysicalVolume* preStepVolume = preStepPoint->GetPhysicalVolume();

	if (preStepPoint->GetStepStatus() != fGeomBoundary )
		return true;

  G4int pdg = track->GetDefinition()->GetPDGEncoding();

//  G4cout << "Hit." << G4endl;

	// static G4RunManager * runManager = G4RunManager::GetRunManager();
 //  const G4Event * event = runManager->GetCurrentEvent();
 //  G4int primaryID = event->GetPrimaryVertex()->GetPrimary()->GetPDGcode();

	G4VPhysicalVolume* physVolume = preStepPoint->GetPhysicalVolume();

	G4double charge = track->GetDefinition()->GetPDGCharge();
	G4double time = preStepPoint->GetGlobalTime();
  G4double kineticEnergy = preStepPoint->GetKineticEnergy();
  G4ThreeVector worldPos = preStepPoint->GetPosition();

  G4TouchableHistory* theTouchable = (G4TouchableHistory*)(preStepPoint->GetTouchable());
  G4ThreeVector hitPosition = 
    theTouchable->GetHistory()->GetTopTransform().TransformPoint(worldPos);
  
  G4ThreeVector HitDirection = preStepPoint->GetMomentumDirection();
  G4double weight = track->GetWeight(); 
  
  class Hit* hit = new class Hit();

  hit->SetEnergy(kineticEnergy);
  hit->SetTime(time);
  hit->SetWeight(weight);
  hit->SetCharge(charge);
  hit->SetPosition(hitPosition);
  hit->SetWorldPosition(worldPos);
  hit->SetDirection(HitDirection);
  hit->SetPDG(pdg);
  hit->SetDetectorID(physVolume->GetCopyNo());

  fHitsCollection->insert(hit);

  aStep->GetTrack()->SetTrackStatus(fStopAndKill);

  return true;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void SensitiveDetector::EndOfEvent(G4HCofThisEvent*)
{

}
